@@ -1,7 +1,7 @@
 /* mpc_get_prec2 -- returns the precisions of the real and of the imaginary
 part through the first two arguments
 
-Copyright (C) 2007 Andreas Enge
+Copyright (C) 2007, 2009 Andreas Enge, Philippe Th\'eveny
 
 This file is part of the MPC Library.
 
@@ -20,11 +20,10 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "mpc.h"
 #include "mpc-impl.h"
 
 void
-mpc_get_prec2 (mp_prec_t *pr, mp_prec_t *pi, mpc_t x)
+mpc_get_prec2 (mp_prec_t *pr, mp_prec_t *pi, mpc_srcptr x)
 {
    *pr = MPFR_PREC (MPC_RE (x));
    *pi = MPFR_PREC (MPC_IM (x));
