@@ -1,6 +1,6 @@
 /* mpc_ui_div -- Divide an unsigned long int by a complex number.
 
-Copyright (C) 2002 Andreas Enge, Paul Zimmermann
+Copyright (C) 2002, 2009 Andreas Enge, Paul Zimmermann, Philippe Th\'eveny
 
 This file is part of the MPC Library.
 
@@ -20,22 +20,20 @@ the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
 #include <limits.h>
-#include "gmp.h"
-#include "mpfr.h"
-#include "mpc.h"
 #include "mpc-impl.h"
 
 int
 mpc_ui_div (mpc_ptr a, unsigned long int b, mpc_srcptr c, mpc_rnd_t rnd)
 {
-  int inex;
+  int inex, inex_im;
   mpc_t bb;
 
   if (mpfr_cmp_ui (MPC_IM(c), 0) == 0) /* c is real */
     {
-      inex = mpfr_ui_div (MPC_RE(a), b, MPC_RE(c), MPC_RND_RE(rnd));
-      mpfr_set_ui (MPC_IM(a), 0, MPC_RND_IM(rnd));
-      return MPC_INEX(inex, 0);
+      inex    = mpfr_ui_div (MPC_RE(a), b, MPC_RE(c), MPC_RND_RE(rnd));
+      /* Let mpfr do the work in case c==0. */
+      inex_im = mpfr_ui_div (MPC_IM(a), 0ul, MPC_IM(c), MPC_RND_IM(rnd));
+      return MPC_INEX(inex, inex_im);
     }
 
   mpc_init2 (bb, sizeof(unsigned long int) * CHAR_BIT);
