@@ -1,6 +1,6 @@
 /* Read data file and check function.
 
-Copyright (C) 2008 Andreas Enge, Philippe Th\'eveny
+Copyright (C) 2008, 2009 Andreas Enge, Philippe Th\'eveny
 
 This file is part of the MPC Library.
 
@@ -21,22 +21,25 @@ MA 02111-1307, USA. */
 
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include "mpc-tests.h"
-#include "mpc-impl.h"
 
-#ifndef __SRCDIR
-#define __SRCDIR .
-#endif
-
-static char *pathname;
-static unsigned long line_number;
+char *pathname;
+unsigned long line_number;
    /* file name with complete path and currently read line;
       kept globally to simplify parameter passing */
-static int nextchar;
+int nextchar;
    /* character appearing next in the file, may be EOF */
 
-const char *mpfr_rnd_mode [] =
+#define MPC_INEX_CMP(r, i, c)                                 \
+  (((r) == TERNARY_NOT_CHECKED || (r) == MPC_INEX_RE(c))      \
+   && ((i) == TERNARY_NOT_CHECKED || (i) == MPC_INEX_IM (c)))
+
+#define MPFR_INEX_STR(inex)                     \
+  (inex) == TERNARY_NOT_CHECKED ? "?"           \
+    : (inex) == +1 ? "+1"                       \
+    : (inex) == -1 ? "-1" : "0"
+
+static const char *mpfr_rnd_mode [] =
   { "GMP_RNDN", "GMP_RNDZ", "GMP_RNDU", "GMP_RNDD" };
 
 const char *rnd_mode[] =
@@ -58,6 +61,42 @@ const char *rnd_mode[] =
     "undefined", "undefined",
   };
 
+/* file functions */
+FILE *
+open_data_file (const char *file_name)
+{
+  FILE *fp;
+  char *src_dir;
+  char default_srcdir[] = ".";
+
+  src_dir = getenv ("srcdir");
+  if (src_dir == NULL)
+    src_dir = default_srcdir;
+
+  pathname = (char *) malloc ((strlen (src_dir)) + strlen (file_name) + 2);
+  if (pathname == NULL)
+    {
+      printf ("Cannot allocate memory\n");
+      exit (1);
+    }
+  sprintf (pathname, "%s/%s", src_dir, file_name);
+  fp = fopen (pathname, "r");
+  if (fp == NULL)
+    {
+      fprintf (stderr, "Unable to open %s\n", pathname);
+      exit (1);
+    }
+
+  return fp;
+}
+
+void
+close_data_file (FILE *fp)
+{
+  free (pathname);
+  fclose (fp);
+}
+
 /* read primitives */
 static void
 skip_line (FILE *fp)
@@ -84,7 +123,7 @@ skip_whitespace (FILE *fp)
    }
 }
 
-static void
+void
 skip_whitespace_comments (FILE *fp)
    /* skips over all whitespace and comments, if any */
 {
@@ -96,10 +135,101 @@ skip_whitespace_comments (FILE *fp)
    }
 }
 
+
+size_t
+read_string (FILE *fp, char **buffer_ptr, size_t buffer_length, const char *name)
+{
+  size_t pos;
+  char *buffer;
+
+  pos = 0;
+  buffer = *buffer_ptr;
+
+  if (nextchar == '"')
+    nextchar = getc (fp);
+  else
+    goto error;
+
+  while (nextchar != EOF && nextchar != '"')
+    {
+      if (nextchar == '\n')
+        line_number++;
+      if (pos + 1 > buffer_length)
+        {
+          buffer = (char *) realloc (buffer, 2 * buffer_length);
+          if (buffer == NULL)
+            {
+              printf ("Cannot allocate memory\n");
+              exit (1);
+            }
+          buffer_length *= 2;
+        }
+      buffer[pos++] = nextchar;
+      nextchar = getc (fp);
+    }
+
+  if (nextchar != '"')
+    goto error;
+
+  if (pos + 1 > buffer_length)
+    {
+      buffer = (char *) realloc (buffer, buffer_length + 1);
+      if (buffer == NULL)
+        {
+          printf ("Cannot allocate memory\n");
+          exit (1);
+        }
+      buffer_length *= 2;
+    }
+  buffer[pos] = '\0';
+
+  nextchar = getc (fp);
+  skip_whitespace_comments (fp);
+
+  buffer_ptr = &buffer;
+
+  return buffer_length;
+
+ error:
+  printf ("Error: Unable to read %s in file '%s' line '%lu'\n",
+          name, pathname, line_number);
+  exit (1);
+}
+
 /* All following read routines skip over whitespace and comments; */
 /* so after calling them, nextchar is either EOF or the beginning */
 /* of a non-comment token.                                        */
-static void
+void
+read_ternary (FILE *fp, int* ternary)
+{
+  switch (nextchar)
+    {
+    case '!':
+      *ternary = TERNARY_ERROR;
+      break;
+    case '?':
+      *ternary = TERNARY_NOT_CHECKED;
+      break;
+    case '+':
+      *ternary = +1;
+      break;
+    case '0':
+      *ternary = 0;
+      break;
+    case '-':
+      *ternary = -1;
+      break;
+    default:
+      printf ("Error: Unexpected ternary value '%c' in file '%s' line %lu\n",
+              nextchar, pathname, line_number);
+      exit (1);
+    }
+
+  nextchar = getc (fp);
+  skip_whitespace_comments (fp);
+}
+
+void
 read_mpfr_rounding_mode (FILE *fp, mpfr_rnd_t* rnd)
 {
   switch (nextchar)
@@ -117,7 +247,7 @@ read_mpfr_rounding_mode (FILE *fp, mpfr_rnd_t* rnd)
       *rnd = GMP_RNDD;
       break;
     default:
-      printf ("Error: Unexpected rounding mode '%c' in file '%s' line %ld\n",
+      printf ("Error: Unexpected rounding mode '%c' in file '%s' line %lu\n",
               nextchar, pathname, line_number);
       exit (1);
     }
@@ -125,23 +255,47 @@ read_mpfr_rounding_mode (FILE *fp, mpfr_rnd_t* rnd)
     nextchar = getc (fp);
     if (nextchar != EOF && !isspace (nextchar)) {
       printf ("Error: Rounding mode not followed by white space in file "
-              "'%s' line %ld\n",
+              "'%s' line %lu\n",
               pathname, line_number);
       exit (1);
     }
     skip_whitespace_comments (fp);
 }
 
-static void
+void
 read_mpc_rounding_mode (FILE *fp, mpc_rnd_t* rnd)
 {
-   int re, im;
+   mpfr_rnd_t re, im;
    read_mpfr_rounding_mode (fp, &re);
    read_mpfr_rounding_mode (fp, &im);
    *rnd = RNDC (re, im);
 }
 
-static mpfr_prec_t
+void
+read_int (FILE *fp, int *nread, const char *name)
+{
+  int n = 0;
+
+  if (nextchar == EOF)
+    {
+      printf ("Error: Unexpected EOF when reading mpfr precision "
+              "in file '%s' line %lu\n",
+              pathname, line_number);
+      exit (1);
+    }
+  ungetc (nextchar, fp);
+  n = fscanf (fp, "%i", nread);
+  if (ferror (fp) || n == 0 || n == EOF)
+    {
+      printf ("Error: Cannot read %s in file '%s' line %lu\n",
+              name, pathname, line_number);
+      exit (1);
+    }
+  nextchar = getc (fp);
+  skip_whitespace_comments (fp);
+}
+
+mpfr_prec_t
 read_mpfr_prec (FILE *fp)
 {
    unsigned long prec;
@@ -149,7 +303,7 @@ read_mpfr_prec (FILE *fp)
 
    if (nextchar == EOF) {
       printf ("Error: Unexpected EOF when reading mpfr precision "
-              "in file '%s' line %ld\n",
+              "in file '%s' line %lu\n",
               pathname, line_number);
       exit (1);
    }
@@ -158,7 +312,7 @@ read_mpfr_prec (FILE *fp)
    if (ferror (fp)) /* then also n == EOF */
       perror ("Error when reading mpfr precision");
    if (n == 0 || n == EOF || prec < MPFR_PREC_MIN || prec > MPFR_PREC_MAX) {
-      printf ("Error: Impossible mpfr precision in file '%s' line %ld\n",
+      printf ("Error: Impossible mpfr precision in file '%s' line %lu\n",
               pathname, line_number);
       exit (1);
    }
@@ -172,14 +326,14 @@ read_mpfr_mantissa (FILE *fp, mpfr_ptr x)
 {
    if (nextchar == EOF) {
       printf ("Error: Unexpected EOF when reading mpfr mantissa "
-              "in file '%s' line %ld\n",
+              "in file '%s' line %lu\n",
               pathname, line_number);
       exit (1);
    }
    ungetc (nextchar, fp);
    if (mpfr_inp_str (x, fp, 0, GMP_RNDN) == 0) {
       printf ("Error: Impossible to read mpfr mantissa "
-              "in file '%s' line %ld\n",
+              "in file '%s' line %lu\n",
               pathname, line_number);
       exit (1);
    }
@@ -187,22 +341,24 @@ read_mpfr_mantissa (FILE *fp, mpfr_ptr x)
    skip_whitespace_comments (fp);
 }
 
-static void
+void
 read_mpfr (FILE *fp, mpfr_ptr x, int *known_sign)
 {
+   int sign;
    mpfr_set_prec (x, read_mpfr_prec (fp));
+   sign = nextchar;
    read_mpfr_mantissa (fp, x);
 
    /* the sign always matters for regular values ('+' is implicit),
       but when no sign appears before 0 or Inf in the data file, it means
       that only absolute value must be checked. */
    if (known_sign != NULL)
-      *known_sign =
-         (!mpfr_zero_p (x) && !mpfr_inf_p (x))
-          || nextchar == '+' || nextchar == '-';
+     *known_sign =
+       (!mpfr_zero_p (x) && !mpfr_inf_p (x))
+       || sign == '+' || sign == '-';
 }
 
-static void
+void
 read_mpc (FILE *fp, mpc_ptr z, known_signs_t *ks)
 {
   read_mpfr (fp, MPC_RE (z), ks == NULL ? NULL : &ks->re);
@@ -211,27 +367,32 @@ read_mpc (FILE *fp, mpc_ptr z, known_signs_t *ks)
 
 /* read lines of data */
 static void
-read_cc (FILE *fp, mpc_ptr expected, known_signs_t *signs, mpc_ptr op,
-         mpc_rnd_t *rnd)
+read_cc (FILE *fp, int *inex_re, int *inex_im, mpc_ptr expected,
+         known_signs_t *signs, mpc_ptr op, mpc_rnd_t *rnd)
 {
+  read_ternary (fp, inex_re);
+  read_ternary (fp, inex_im);
   read_mpc (fp, expected, signs);
   read_mpc (fp, op, NULL);
   read_mpc_rounding_mode (fp, rnd);
 }
 
 static void
-read_fc (FILE *fp, mpfr_ptr expected, int *sign, mpc_ptr op,
+read_fc (FILE *fp, int *inex, mpfr_ptr expected, int *sign, mpc_ptr op,
          mpfr_rnd_t *rnd)
 {
+  read_ternary (fp, inex);
   read_mpfr (fp, expected, sign);
   read_mpc (fp, op, NULL);
   read_mpfr_rounding_mode (fp, rnd);
 }
 
 static void
-read_ccc (FILE *fp, mpc_ptr expected, known_signs_t *signs,
-          mpc_ptr op1, mpc_ptr op2, mpc_rnd_t *rnd)
+read_ccc (FILE *fp, int *inex_re, int *inex_im, mpc_ptr expected,
+          known_signs_t *signs, mpc_ptr op1, mpc_ptr op2, mpc_rnd_t *rnd)
 {
+  read_ternary (fp, inex_re);
+  read_ternary (fp, inex_im);
   read_mpc (fp, expected, signs);
   read_mpc (fp, op1, NULL);
   read_mpc (fp, op2, NULL);
@@ -239,10 +400,12 @@ read_ccc (FILE *fp, mpc_ptr expected, known_signs_t *signs,
 }
 
 static void
-read_cfc (FILE *fp, mpc_ptr expected, known_signs_t *signs, mpfr_ptr op1,
-          mpc_ptr op2, mpc_rnd_t *rnd)
+read_cfc (FILE *fp, int *inex_re, int *inex_im, mpc_ptr expected,
+          known_signs_t *signs, mpfr_ptr op1, mpc_ptr op2, mpc_rnd_t *rnd)
 {
 
+  read_ternary (fp, inex_re);
+  read_ternary (fp, inex_im);
   read_mpc (fp, expected, signs);
   read_mpfr (fp, op1, NULL);
   read_mpc (fp, op2, NULL);
@@ -250,9 +413,11 @@ read_cfc (FILE *fp, mpc_ptr expected, known_signs_t *signs, mpfr_ptr op1,
 }
 
 static void
-read_ccf (FILE *fp, mpc_ptr expected, known_signs_t *signs, mpc_ptr op1,
-          mpfr_ptr op2, mpc_rnd_t *rnd)
+read_ccf (FILE *fp, int *inex_re, int *inex_im, mpc_ptr expected,
+          known_signs_t *signs, mpc_ptr op1, mpfr_ptr op2, mpc_rnd_t *rnd)
 {
+  read_ternary (fp, inex_re);
+  read_ternary (fp, inex_im);
   read_mpc (fp, expected, signs);
   read_mpc (fp, op1, NULL);
   read_mpfr (fp, op2, NULL);
@@ -266,56 +431,46 @@ data_check (mpc_function function, const char *file_name)
 {
   FILE *fp;
 
+  int inex_re;
   mpfr_t x1, x2;
   mpfr_rnd_t mpfr_rnd = GMP_RNDN;
   int sign_real;
+
+  int inex_im;
   mpc_t z1, z2, z3, z4;
   mpc_rnd_t rnd = MPC_RNDNN;
   known_signs_t signs;
+  int inex = 0;
 
-  /* 1. open data file */
-  pathname = (char *)malloc ((strlen (QUOTE (__SRCDIR)) + strlen (file_name)
-                              + 2) * sizeof (char));
-  if (pathname == NULL)
-    {
-      printf ("Cannot allocate memory\n");
-      exit (1);
-    }
-  sprintf (pathname, QUOTE (__SRCDIR)"/%s", file_name);
-  fp = fopen (pathname, "r");
-  if (fp == NULL)
-    {
-      fprintf (stderr, "Unable to open %s\n", pathname);
-      exit (1);
-    }
+  fp = open_data_file (file_name);
 
-  /* 2. init needed variables */
-  mpc_init (z1);
+  /* 1. init needed variables */
+  mpc_init2 (z1, 2);
   switch (function.type)
     {
     case FC:
       mpfr_init (x1);
       mpfr_init (x2);
       break;
-    case CC: case V_CC:
-      mpc_init (z2);
-      mpc_init (z3);
+    case CC:
+      mpc_init2 (z2, 2);
+      mpc_init2 (z3, 2);
       break;
     case CCC:
-      mpc_init (z2);
-      mpc_init (z3);
-      mpc_init (z4);
+      mpc_init2 (z2, 2);
+      mpc_init2 (z3, 2);
+      mpc_init2 (z4, 2);
       break;
     case CFC: case CCF:
       mpfr_init (x1);
-      mpc_init (z2);
-      mpc_init (z3);
+      mpc_init2 (z2, 2);
+      mpc_init2 (z3, 2);
       break;
     default:
       ;
     }
 
-  /* 3. read data file */
+  /* 2. read data file */
   line_number = 1;
   nextchar = getc (fp);
   skip_whitespace_comments (fp);
@@ -327,10 +482,11 @@ data_check (mpc_function function, const char *file_name)
       switch (function.type)
         {
         case FC:
-          read_fc (fp, x1, &sign_real, z1, &mpfr_rnd);
+          read_fc (fp, &inex_re, x1, &sign_real, z1, &mpfr_rnd);
           mpfr_set_prec (x2, MPFR_PREC (x1));
-          function.pointer.FC (x2, z1, mpfr_rnd);
-          if (!same_mpfr_value (x1, x2, sign_real))
+          inex = function.pointer.FC (x2, z1, mpfr_rnd);
+          if ((inex_re != TERNARY_NOT_CHECKED && inex_re != inex)
+              || !same_mpfr_value (x1, x2, sign_real))
             {
               mpfr_t got, expected;
               mpc_t op;
@@ -338,8 +494,11 @@ data_check (mpc_function function, const char *file_name)
               got[0] = x2[0];
               expected[0] = x1[0];
               printf ("%s(op) failed (%s:%lu)\nwith rounding mode %s\n",
-                      function.name, file_name, line_number,
+                      function.name, file_name, line_number-1,
                       mpfr_rnd_mode[mpfr_rnd]);
+              if (inex_re != TERNARY_NOT_CHECKED && inex_re != inex)
+                printf("ternary value: got %s, expected %s\n",
+                       MPFR_INEX_STR (inex), MPFR_INEX_STR (inex_re));
               OUT (op);
               printf ("     ");
               MPFR_OUT (got);
@@ -350,40 +509,23 @@ data_check (mpc_function function, const char *file_name)
           break;
 
         case CC:
-          read_cc (fp, z1, &signs, z2, &rnd);
+          read_cc (fp, &inex_re, &inex_im, z1, &signs, z2, &rnd);
           mpfr_set_prec (MPC_RE (z3), MPC_PREC_RE (z1));
           mpfr_set_prec (MPC_IM (z3), MPC_PREC_IM (z1));
-          function.pointer.CC (z3, z2, rnd);
-          if (!same_mpc_value (z3, z1, signs))
+          inex = function.pointer.CC (z3, z2, rnd);
+          if (!MPC_INEX_CMP (inex_re, inex_im, inex)
+              || !same_mpc_value (z3, z1, signs))
             {
               mpc_t op, got, expected; /* display sensible variable names */
               op[0] = z2[0];
               expected[0]= z1[0];
               got[0] = z3[0];
               printf ("%s(op) failed (line %lu)\nwith rounding mode %s\n",
-                      function.name, line_number, rnd_mode[rnd]);
-              OUT (op);
-              printf ("     ");
-              OUT (got);
-              OUT (expected);
-
-              exit (1);
-            }
-          break;
-
-        case V_CC:
-          read_cc (fp, z1, &signs, z2, &rnd);
-          mpfr_set_prec (MPC_RE (z3), MPC_PREC_RE (z1));
-          mpfr_set_prec (MPC_IM (z3), MPC_PREC_IM (z1));
-          function.pointer.V_CC (z3, z2, rnd);
-          if (!same_mpc_value (z3, z1, signs))
-            {
-              mpc_t op, got, expected; /* display sensible variable names */
-              op[0] = z2[0];
-              expected[0]= z1[0];
-              got[0] = z3[0];
-              printf ("%s(op) failed (line %lu)\nwith rounding mode %s\n      ",
-                      function.name, line_number, rnd_mode[rnd]);
+                      function.name, line_number-1, rnd_mode[rnd]);
+              if (!MPC_INEX_CMP (inex_re, inex_im, inex))
+                printf("ternary value: got %s, expected (%s, %s)\n",
+                       MPC_INEX_STR (inex),
+                       MPFR_INEX_STR (inex_re), MPFR_INEX_STR (inex_im));
               OUT (op);
               printf ("     ");
               OUT (got);
@@ -394,11 +536,12 @@ data_check (mpc_function function, const char *file_name)
           break;
 
         case CCC:
-          read_ccc (fp, z1, &signs, z2, z3, &rnd);
+          read_ccc (fp, &inex_re, &inex_im, z1, &signs, z2, z3, &rnd);
           mpfr_set_prec (MPC_RE(z4), MPC_PREC_RE (z1));
           mpfr_set_prec (MPC_IM(z4), MPC_PREC_IM (z1));
-          function.pointer.CCC (z4, z2, z3, rnd);
-          if (!same_mpc_value (z4, z1, signs))
+          inex = function.pointer.CCC (z4, z2, z3, rnd);
+          if (!MPC_INEX_CMP (inex_re, inex_im, inex)
+              || !same_mpc_value (z4, z1, signs))
             {
               /* display sensible variable names */
               mpc_t op1, op2, got, expected;
@@ -407,7 +550,11 @@ data_check (mpc_function function, const char *file_name)
               expected[0]= z1[0];
               got[0] = z4[0];
               printf ("%s(op) failed (line %lu)\nwith rounding mode %s\n",
-                      function.name, line_number, rnd_mode[rnd]);
+                      function.name, line_number-1, rnd_mode[rnd]);
+              if (!MPC_INEX_CMP (inex_re, inex_im, inex))
+                printf("ternary value: got %s, expected (%s, %s)\n",
+                       MPC_INEX_STR (inex),
+                       MPFR_INEX_STR (inex_re), MPFR_INEX_STR (inex_im));
               OUT (op1);
               OUT (op2);
               printf ("     ");
@@ -418,8 +565,9 @@ data_check (mpc_function function, const char *file_name)
             }
           if (function.properties & FUNC_PROP_SYMETRIC)
             {
-              function.pointer.CCC (z4, z3, z2, rnd);
-              if (!same_mpc_value (z4, z1, signs))
+              inex = function.pointer.CCC (z4, z3, z2, rnd);
+              if (!MPC_INEX_CMP (inex_re, inex_im, inex)
+              || !same_mpc_value (z4, z1, signs))
                 {
                   /* display sensible variable names */
                   mpc_t op1, op2, got, expected;
@@ -429,7 +577,11 @@ data_check (mpc_function function, const char *file_name)
                   got[0] = z4[0];
                   printf ("%s(op) failed (line %lu/symetric test)\n"
                           "with rounding mode %s\n",
-                          function.name, line_number, rnd_mode[rnd]);
+                          function.name, line_number-1, rnd_mode[rnd]);
+                  if (!MPC_INEX_CMP (inex_re, inex_im, inex))
+                    printf("ternary value: got %s, expected (%s, %s)\n",
+                           MPC_INEX_STR (inex),
+                           MPFR_INEX_STR (inex_re), MPFR_INEX_STR (inex_im));
                   OUT (op1);
                   OUT (op2);
                   printf ("     ");
@@ -442,11 +594,12 @@ data_check (mpc_function function, const char *file_name)
           break;
 
         case CFC:
-          read_cfc (fp, z1, &signs, x1, z2, &rnd);
+          read_cfc (fp, &inex_re, &inex_im, z1, &signs, x1, z2, &rnd);
           mpfr_set_prec (MPC_RE(z3), MPC_PREC_RE (z1));
           mpfr_set_prec (MPC_IM(z3), MPC_PREC_IM (z1));
-          function.pointer.CFC (z3, x1, z2, rnd);
-          if (!same_mpc_value (z3, z1, signs))
+          inex = function.pointer.CFC (z3, x1, z2, rnd);
+          if (!MPC_INEX_CMP (inex_re, inex_im, inex)
+              || !same_mpc_value (z3, z1, signs))
             {
               /* display sensible variable names */
               mpc_t op2, got, expected;
@@ -456,7 +609,11 @@ data_check (mpc_function function, const char *file_name)
               expected[0]= z1[0];
               got[0] = z3[0];
               printf ("%s(op) failed (line %lu)\nwith rounding mode %s\n",
-                      function.name, line_number, rnd_mode[rnd]);
+                      function.name, line_number-1, rnd_mode[rnd]);
+              if (!MPC_INEX_CMP (inex_re, inex_im, inex))
+                printf("ternary value: got %s, expected (%s, %s)\n",
+                       MPC_INEX_STR (inex),
+                       MPFR_INEX_STR (inex_re), MPFR_INEX_STR (inex_im));
               MPFR_OUT (op1);
               OUT (op2);
               printf ("     ");
@@ -468,11 +625,12 @@ data_check (mpc_function function, const char *file_name)
           break;
 
         case CCF:
-          read_ccf (fp, z1, &signs, z2, x1, &rnd);
+          read_ccf (fp, &inex_re, &inex_im, z1, &signs, z2, x1, &rnd);
           mpfr_set_prec (MPC_RE(z3), MPC_PREC_RE (z1));
           mpfr_set_prec (MPC_IM(z3), MPC_PREC_IM (z1));
-          function.pointer.CCF (z3, z2, x1, rnd);
-          if (!same_mpc_value (z3, z1, signs))
+          inex = function.pointer.CCF (z3, z2, x1, rnd);
+          if (!MPC_INEX_CMP (inex_re, inex_im, inex)
+              || !same_mpc_value (z3, z1, signs))
             {
               /* display sensible variable names */
               mpc_t op1, got, expected;
@@ -482,7 +640,11 @@ data_check (mpc_function function, const char *file_name)
               expected[0]= z1[0];
               got[0] = z3[0];
               printf ("%s(op) failed (line %lu)\nwith rounding mode %s\n",
-                      function.name, line_number, rnd_mode[rnd]);
+                      function.name, line_number-1, rnd_mode[rnd]);
+              if (!MPC_INEX_CMP (inex_re, inex_im, inex))
+                printf("ternary value: got %s, expected (%s, %s)\n",
+                       MPC_INEX_STR (inex),
+                       MPFR_INEX_STR (inex_re), MPFR_INEX_STR (inex_im));
               OUT (op1);
               MPFR_OUT (op2);
               printf ("     ");
@@ -498,7 +660,7 @@ data_check (mpc_function function, const char *file_name)
         }
     }
 
-  /* 4. Clear used variables */
+  /* 3. Clear used variables */
   mpc_clear (z1);
   switch (function.type)
     {
@@ -506,7 +668,7 @@ data_check (mpc_function function, const char *file_name)
       mpfr_clear (x1);
       mpfr_clear (x2);
       break;
-    case CC: case V_CC:
+    case CC:
       mpc_clear (z2);
       mpc_clear (z3);
       break;
@@ -524,6 +686,5 @@ data_check (mpc_function function, const char *file_name)
       ;
     }
 
-  fclose (fp);
-  free (pathname);
+  close_data_file (fp);
 }
